import { world, system } from '@minecraft/server'

system.runInterval(() => {
  world.getDimension("minecraft:overworld").getPlayers().forEach(player => {
    if (world.getTimeOfDay() < 23000 && world.getTimeOfDay() > 13000 && world.getMoonPhase() == 0 && player.matches({ excludeTags: ["hell_moon"] })) {
      player.sendMessage({ "rawtext": [{ "translate": "alert.hell_moon_rising" }] });
      player.addTag('hell_moon');
      player.runCommandAsync(`fog @a push "minecraft:fog_hell" hmoon`)
    }
    if (world.getTimeOfDay() > 23000 && player.matches({ tags: ["hell_moon"] })) {
      player.sendMessage({ "rawtext": [{ "translate": "safe.hell_moon_gone" }] });
      player.removeTag('hell_moon')
      player.runCommandAsync(`fog @a remove hmoon`)
    }
  });
  world.getDimension("minecraft:overworld").getEntities().forEach(entity => {
    if (world.getTimeOfDay() > 23000 && entity.matches({ tags: ["hell_moon_creature"] })) {
      entity.remove();
    }
  });
}, 20);

world.beforeEvents.playerInteractWithBlock.subscribe((e) => {
  const player = e.player;
  const block = e.block;
  if (block && block.typeId === "minecraft:bed") {
    const hellMoon = world.getMoonPhase() == 0;
    if (hellMoon && player.matches({ tags: ["hell_moon"] })) {
      player.sendMessage({ "rawtext": [{ translate: "alert.hell_moon_bed" }] })
      e.cancel = true;
    }
  }
});

world.afterEvents.entitySpawn.subscribe(e => {
  const entity = e.entity;
  const night = world.getTimeOfDay() < 23000 && world.getTimeOfDay() > 13000;
  const hellMoon = world.getMoonPhase() == 0;
  if (hellMoon && night) {
    const weight = Math.round(Math.random() * 4);
    if (entity.matches({ families: ["monster"] })) {
      entity.addTag("hell_moon_creature");
      if (weight == 1) {
        entity.addTag("speeder");
      }
      if (weight == 2) {
        entity.addTag("poisoner");
      }
      if (weight == 3) {
        entity.addTag("brute")
      }
      if (weight == 4) {
        entity.addTag("wither")
      }
    }
  }
});

world.afterEvents.entityHitEntity.subscribe((e) => {
  const attacker = e.damagingEntity;
  const victim = e.hitEntity
  if (attacker.matches({ tags: ["poisoner"] })) {
    victim.addEffect("minecraft:poison", 150, { amplifier: 2 });
  }
  if (attacker.matches({ tags: ["wither"] })) {
    victim.addEffect("minecraft:wither", 250, { amplifier: 2 });
    victim.addTag("wither_infected");
  }
  if (attacker.matches({ tags: ["brute"] })) {
    const viewDirection = attacker.getViewDirection();
    victim.applyKnockback(5 * viewDirection.x, 5 * viewDirection.z, 0.1, 0.4);
  }
});

world.afterEvents.entityDie.subscribe((e) => {
  if (e.deadEntity.typeId === 'minecraft:player' && e.deadEntity.hasTag('wither_infected')) {
    e.deadEntity.runCommand(`summon minecraft:wither_skeleton`);
    e.deadEntity.removeTag("wither_infected");
  }
});